"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.askStringIdQuestions = exports.stringIdQuestions = exports.questionIdAttrName = exports.questionIdPositionInEdit = exports.questionIdPositionInList = exports.questionShowIdInList = void 0;
const questions_1 = require("../../../../common/questions");
// The view in the exported project model never contains the ID attribute.
// Therefore we are asking whether the ID attribute shall be shown in the List component.
// For StringId entities we always show the ID field in the Edit component
// as it is required to create the entities.
exports.questionShowIdInList = {
    code: 'listShowIdAttr',
    caption: 'Show ID attribute in the List component?',
    propertyType: "BOOLEAN" /* BOOLEAN */,
    required: true
};
exports.questionIdPositionInList = {
    code: 'listIdAttrPos',
    caption: 'Position of the ID attribute in the List component \n' +
        '(behaves like an array index, e.g. enter 0 for the ID to appear as the first row/column, \n' +
        '-1 for next-to-last, etc.).',
    propertyType: "INTEGER" /* INTEGER */,
    required: true
};
exports.questionIdPositionInEdit = {
    code: 'editIdAttrPos',
    caption: 'Position of the ID attribute in the Edit component \n' +
        '(behaves like an array index, e.g. enter 0 for the ID to appear as the first field of the form, \n' +
        '-1 for next-to-last, etc.).',
    propertyType: "INTEGER" /* INTEGER */,
    required: true
};
exports.questionIdAttrName = {
    code: 'idAttrName',
    caption: 'You have chosen a String ID entity, but we could not find the ID attribute name in the project model. \n' +
        'What is the ID attribute name?',
    propertyType: "STRING" /* STRING */,
    required: true
};
exports.stringIdQuestions = [
    exports.questionShowIdInList,
    exports.questionIdPositionInList,
    exports.questionIdPositionInEdit,
    exports.questionIdAttrName
];
async function askStringIdQuestions(entity, projectModel, gen, questions = {}) {
    const { idAttrNameQuestion = exports.questionIdAttrName, listShowIdQuestion = exports.questionShowIdInList, listIdPositionQuestion = exports.questionIdPositionInList, editIdPositionQuestion = exports.questionIdPositionInEdit } = questions;
    let idAttrNameAnswers = {};
    if (entity.idAttributeName == null) {
        // Seems that the project model was created with an older version of Studio.
        // Ask for ID attribute name.
        idAttrNameAnswers = await gen.prompt(questions_1.fromStudioProperties([idAttrNameQuestion], projectModel));
    }
    // Ask whether to show the ID field in the List component
    const showIdAnswers = await gen.prompt(questions_1.fromStudioProperties([listShowIdQuestion], projectModel));
    // Ask at which position to show the ID field in the List and Edit components
    const idPositionQuestions = [];
    if (showIdAnswers.listShowIdAttr) {
        idPositionQuestions.push(listIdPositionQuestion);
    }
    if (editIdPositionQuestion !== false) {
        idPositionQuestions.push(editIdPositionQuestion);
    }
    let idPositionAnswers = {};
    if (idPositionQuestions.length > 0) {
        idPositionAnswers = await gen.prompt(questions_1.fromStudioProperties(idPositionQuestions, projectModel));
    }
    return { ...idAttrNameAnswers, ...showIdAnswers, ...idPositionAnswers };
}
exports.askStringIdQuestions = askStringIdQuestions;
//# sourceMappingURL=stringId.js.map