"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.addI18nMessagesEn = exports.addScreenI18nKeyEn = exports.writeAmplicodeComponent = void 0;
const path_1 = __importDefault(require("path"));
const utils_1 = require("../../../../../common/utils");
const addMvpAppMenu_1 = require("./addMvpAppMenu");
const previews_registration_1 = require("../previews-registration");
async function writeAmplicodeComponent(templateModel, gen, srcPath) {
    const { relDirShift, componentName, route, shouldAddToMenu, menuItemName } = templateModel;
    gen.log(`Generating to ${gen.destinationPath()}`);
    gen.fs.copyTpl(srcPath, gen.destinationPath(componentName + '.tsx'), templateModel);
    gen.log(`✓ Component ${componentName} written`);
    if (shouldAddToMenu) {
        addMvpAppMenu_1.addMvpAppMenu({
            gen,
            dirShift: relDirShift,
            route,
            componentName,
            menuItemName
        });
    }
    else {
        gen.log(`✓ Menu item not required for ${componentName}`);
    }
    addScreenI18nKeyEn(componentName, relDirShift, gen);
    gen.log(`✓ i18n messages added for ${componentName}`);
    previews_registration_1.addComponentPreviews(gen, relDirShift, componentName, componentName);
    gen.log(`✓ Component previews added for ${componentName}`);
}
exports.writeAmplicodeComponent = writeAmplicodeComponent;
function addScreenI18nKeyEn(className, dirShift, gen, componentType = "screen") {
    const existingMessagesPath = path_1.default.join(dirShift, 'core', 'i18n', 'messages', 'en.json');
    const existingMessages = gen.fs.readJSON(existingMessagesPath);
    if (existingMessages == null) {
        throw new Error('i18n messages not found');
    }
    const screenNameKey = `${componentType}.${className}`;
    if (Object.keys(existingMessages).includes(screenNameKey)) {
        return;
    }
    const screenCaption = utils_1.splitByCapitalLetter(utils_1.capitalizeFirst(className));
    const mergedMessages = {
        ...existingMessages,
        [screenNameKey]: screenCaption
    };
    gen.fs.writeJSON(existingMessagesPath, mergedMessages);
}
exports.addScreenI18nKeyEn = addScreenI18nKeyEn;
function addI18nMessagesEn(dirShift, gen, messages) {
    const existingMessagesPath = path_1.default.join(dirShift, 'core', 'i18n', 'messages', 'en.json');
    const existingMessages = gen.fs.readJSON(existingMessagesPath);
    if (existingMessages == null) {
        throw new Error('i18n messages not found');
    }
    const existingKeys = Object.keys(existingMessages);
    const messagesKeys = Object.keys(messages);
    const keysToMerge = messagesKeys.filter((messageKey) => {
        const isMessageKeyExist = existingKeys.includes(messageKey);
        if (isMessageKeyExist) {
            gen.log(`message key '${messageKey}' already exists and will not be merged`);
        }
        return !isMessageKeyExist;
    });
    if (keysToMerge.length === 0) {
        gen.log(`messages don't have any keys to merge`);
        return;
    }
    const messagesToMerge = keysToMerge.reduce((msgs, key) => {
        return {
            ...msgs,
            [key]: messages[key]
        };
    }, {});
    const mergedMessages = {
        ...existingMessages,
        ...messagesToMerge
    };
    gen.fs.writeJSON(existingMessagesPath, mergedMessages);
}
exports.addI18nMessagesEn = addI18nMessagesEn;
//# sourceMappingURL=mvp.js.map