"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.replaceStringValue = exports.replaceStringValueInFile = void 0;
const jscodeshift_1 = __importDefault(require("jscodeshift"));
const ReplacementTargetNotFoundError_1 = require("./ReplacementTargetNotFoundError");
async function replaceStringValueInFile(gen, file, variableName, newValue) {
    const filePath = gen.destinationPath(file);
    const source = gen.fs.read(filePath);
    const tsParser = jscodeshift_1.default.withParser('ts');
    const ast = tsParser(source);
    try {
        replaceStringValue(ast, variableName, newValue);
    }
    catch (error) {
        if (error instanceof ReplacementTargetNotFoundError_1.ReplacementTargetNotFoundError) {
            gen.log(`WARNING: Was trying to change ${variableName} in "${file}" to ${newValue}, but this file does not contain this variable. This may break the application.`);
            return;
        }
    }
    const output = ast.toSource();
    gen.fs.write(filePath, output);
}
exports.replaceStringValueInFile = replaceStringValueInFile;
function replaceStringValue(ast, variableName, newValue) {
    const loginUriStringValue = ast
        .findVariableDeclarators(variableName)
        .find(jscodeshift_1.default.StringLiteral);
    if (loginUriStringValue.length === 0) {
        throw new ReplacementTargetNotFoundError_1.ReplacementTargetNotFoundError();
    }
    loginUriStringValue
        .replaceWith(jscodeshift_1.default.stringLiteral(newValue));
}
exports.replaceStringValue = replaceStringValue;
//# sourceMappingURL=replaceStringValue.js.map